import base64
from typing import Optional, Any, List

from pytoncenter import AsyncTonCenterClientV3
from pytoncenter.v3.models import RunGetMethodRequest, ExternalMessage

from ._base import Client


class ToncenterClient(Client):
    def __init__(
            self,
            api_key: str,
            is_testnet: Optional[bool] = False,
    ) -> None:
        self.client = AsyncTonCenterClientV3(
            network="testnet" if is_testnet else "mainnet",
            api_key=api_key,
        )

    async def run_get_method(
            self,
            address: str,
            method_name: str,
            stack: Optional[List[Any]] = None,
    ) -> Any:
        req = RunGetMethodRequest(
            address=address,
            method=method_name,
            stack=stack or [],
        )

        return await self.client.run_get_method(req)

    async def send_message(self, boc: str) -> None:
        req = ExternalMessage(boc=base64.b64encode(bytes.fromhex(boc)).decode())
        await self.client.send_message(req)
